package com.atguigu.tingshu.order.api;

import com.atguigu.tingshu.common.login.GuiGuLogin;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.model.order.OrderInfo;
import com.atguigu.tingshu.order.service.OrderInfoService;
import com.atguigu.tingshu.vo.order.OrderInfoVo;
import com.atguigu.tingshu.vo.order.TradeVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@Tag(name = "订单管理")
@RestController
@RequestMapping("api/order")
@SuppressWarnings({"all"})
public class OrderInfoApiController {

    @Autowired
    private OrderInfoService orderInfoService;


    /**
     * 对购买商品（VIP会员、专辑、声音）进行结算，渲染结算页
     *
     * @param tradeVo
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "对购买商品（VIP会员、专辑、声音）进行结算，渲染结算页")
    @PostMapping("/orderInfo/trade")
    public Result<OrderInfoVo> trade(@RequestBody @Validated TradeVo tradeVo) {
        //1.获取用户ID
        Long userId = AuthContextHolder.getUserId();
        //2.调用业务逻辑处理不同商品类别结算页数据
        OrderInfoVo orderInfoVo = orderInfoService.trade(userId, tradeVo);
        return Result.ok(orderInfoVo);
    }


    /**
     * 提交订单
     *
     * @param orderInfoVo
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "提交订单")
    @PostMapping("/orderInfo/submitOrder")
    public Result<Map<String, String>> submitOrder(@RequestBody @Validated OrderInfoVo orderInfoVo) {
        //1.获取用户ID
        Long userId = AuthContextHolder.getUserId();
        //2.调用业务逻辑处理订单提交
        String orderNo = orderInfoService.submitOrder(userId, orderInfoVo);
        //3.返回对象封装订单编号
        Map<String, String> map = new HashMap<>();
        map.put("orderNo", orderNo);
        return Result.ok(map);
    }


    /**
     * 根据订单编号查询订单信息
     * TODO 场景：如果是提供给用户端需要登录才能访问(订单编号+用户ID)；如果是系统内部调用，则不需要登录
     *
     * @param orderNo
     * @return
     */
    @Operation(summary = "根据订单编号查询订单信息")
    @GetMapping("/orderInfo/getOrderInfo/{orderNo}")
    public Result<OrderInfo> getOrderInfo(@PathVariable("orderNo") String orderNo) {
        OrderInfo orderInfo = orderInfoService.getOrderInfo(orderNo);
        return Result.ok(orderInfo);
    }

    /**
     * 分页查询当前用户订单列表
     *
     * @param page
     * @param limit
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "分页查询当前用户订单列表(包含订单明细)")
    @GetMapping("/orderInfo/findUserPage/{page}/{limit}")
    public Result<Page<OrderInfo>> findUserOrderPage(@PathVariable Long page, @PathVariable Long limit) {
        //1.获取用户ID
        Long userId = AuthContextHolder.getUserId();
        //2.创建分页对象Page，封装当前页，页大小
        Page<OrderInfo> pageInfo = new Page<>(page, limit);
        //3.调用业务层分页方法
        pageInfo = orderInfoService.findUserOrderPage(pageInfo, userId);
        return Result.ok(pageInfo);
    }


    /**
     * 用户付款成功后更新订单状态以及虚拟物品发货
     *
     * @param orderNo
     * @return
     */
    @Operation(summary = "用户付款成功后更新订单状态以及虚拟物品发货")
    @GetMapping("/orderInfo/orderPaySuccess/{orderNo}")
    public Result orderPaySuccess(@PathVariable("orderNo") String orderNo) {
        orderInfoService.orderPaySuccess(orderNo);
        return Result.ok();
    }
}

